/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.mc;

import ch.hortis.sonar.model.Collectable;
import ch.hortis.sonar.model.File;
import ch.hortis.sonar.model.RuleFailure;
import ch.hortis.sonar.model.RuleFailureLevel;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.List;

public class CheckstyleCollector extends BaseMeasuresCollector {

  private XmlReportParser reportParser;
  private List<String> sourceDirs;

  public boolean initialize(MavenProject project) {
    java.io.File report = findFileFromBuildDirectory(project, "checkstyle-result.xml");
    boolean ok = false;
    if (report != null) {
      ok = true;
      sourceDirs = project.getCompileSourceRoots();
      reportParser = new XmlReportParser();
      reportParser.parse(report);
    }
    return ok;
  }

  public List<Collectable> collect() throws MojoExecutionException {
    return collectRuleFailures();
  }

  public List<Collectable> collectRuleFailures() throws MojoExecutionException {
    List<Collectable> failures = new ArrayList<Collectable>(4096);
    NodeList files = reportParser.executeXPathNodeList("/checkstyle/file");
    if (files != null) {
      for (int i = 0; i < files.getLength(); i++) {
        Element element = (Element) files.item(i);
        String checkstyleName = element.getAttribute("name");

        List<Element> errors = reportParser.getChildElements(element, "error");
        for (Element error : errors) {
          RuleFailure failure = new RuleFailure();
          String source = error.getAttribute("source");
          if (rulesService.getRuleByPluginKey(source) == null) {
            continue;
          }
          failure.setRule(rulesService.getRuleByPluginKey(source));
          failure.setFile(getFile(checkstyleName));
          failure.setMessage(error.getAttribute("message"));
          String line = error.getAttribute("line");
          if (line != null && !"".equals(line)) {
            failure.addParameter("line", Double.valueOf(line));
          }
          String column = error.getAttribute("column");
          if (column != null && !"".equals(column)) {
            failure.addParameter("column", Double.valueOf(column));
          }
          String severity = error.getAttribute("severity");
          if ("error".equals(severity)) {
            failure.setLevel(RuleFailureLevel.ERROR);
          } else if ("warning".equals(severity)) {
            failure.setLevel(RuleFailureLevel.WARNING);
          } else {
            failure.setLevel(RuleFailureLevel.INFO);
          }

          failures.add(failure);
        }
      }
    }
    return failures;
  }

  private File getFile(String checkstyleFilename) {
    checkstyleFilename = checkstyleFilename.replace('\\', '/');
    String filename = StringUtils.substringAfterLast(checkstyleFilename, "/");
    String namespace = StringUtils.substringBeforeLast(checkstyleFilename, filename);
    for (String sourceDir : sourceDirs) {
      sourceDir = sourceDir.replace('\\', '/');
      if (!sourceDir.endsWith("/")) {
        sourceDir = sourceDir += "/";
      }
      if (namespace.startsWith(sourceDir) || namespace.contains(sourceDir)) {
        namespace = StringUtils.substringAfter(namespace, sourceDir).replace('/', '.');
        if (namespace.endsWith(".")) {
          namespace = StringUtils.chop(namespace);
        }
        break;
      }
    }

    return getFilesRepository().getFile(namespace, filename);
  }
}