/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn.mc;

import ch.hortis.sonar.model.Collectable;
import ch.hortis.sonar.service.MetricService;
import ch.hortis.sonar.service.RulesService;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

import java.util.List;

/**
 * Base interface of a report plugins metrics collector, the metric collector has multiple life cycle phases
 */
public interface MeasuresCollector {

  public boolean initialize(MavenProject project, MetricService metricService,
                         RulesService rulesService,
                         FilesRepository filesRepository);

  /**
   * collect metrics for a given maven project and report plugin. The scope of the collected metrics is at the project level
   * Meaning that the returned metrics should represent a sum of metrics data for the whole project.
   *
   * @return a collection of project metrics for the given report plugin or null if no metrics can be collected
   * @throws MojoExecutionException if some errors occurs such as parsing errors.
   */
  public List<Collectable> collect() throws MojoExecutionException;
}
